package acg.prototype;

import acg.prototype.content.Route;
import acg.prototype.db.DbAdapter;
import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.text.method.PasswordTransformationMethod;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

/**
 * The main activity of the application.
 */
public class Main extends Activity {

	/**
	 * The route the user has been assigned.
	 */
	private Route route = null;

	/**
	 * The currentTaskId of the route.
	 */
	private int currentTaskId;

	/**
	 * The dbAdapter for connecting to the database.
	 */
	private DbAdapter dbAdapter;

	private static final int RESET_ID = Menu.FIRST;
	private static final int RESET_DIALOG = 1;
	private static final String PASSWORD = "hopp";
	
	/**
	 * TextView representing the accumulated penalties. 
	 */
	private TextView penalties;
	
	/**
	 * TextView representing the progress in the game.
	 */
	private TextView progress;

	/**
	 * Called when the application is started.
	 * 
	 * Immediately starts a new activity with the UI for the current task.
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		createDatabase();

		loadGameState();

		createWelcomeScreen();
	}

	/**
	 * Loads the current game state or creates one if the game has just started.
	 */
	private void loadGameState() {
		// Loads the route
		if (route == null) {
			// Loads the route id from the database
			dbAdapter.open();
			int id = dbAdapter.getGameStateRoute();
			dbAdapter.close();

			// Creates a new route if there is no route in game state table,
			// else its loads the route stored in the game state table
			if (id != 0) {
				Log.e("Database", "Henter ut routeId =  " + id);
				route = new Route(dbAdapter, id);
			} else {
				Log.e("Database", "Lager ny route");
				route = new Route(dbAdapter);
			}
		}
		// Loads the current task according to the game state table
		dbAdapter.open();
		currentTaskId = dbAdapter.getGameStateTaskId();
		Log.e("Database", "Henter ut currentTaskId = " + currentTaskId);

		// Loads the first task if there is no current task in the game state
		// table
		if (currentTaskId == 0) {
			currentTaskId = route.getFirstTaskId();
		}

		// Updates the game state, there are changes in the game state if the
		// currentTaskId is 0 or route is null
		dbAdapter.updateGameState(route.getRouteId(), currentTaskId);
		dbAdapter.close();

	}

	/**
	 * Creates the database (it is only run one time for each installation of
	 * the app).
	 */
	private void createDatabase() {
		dbAdapter = new DbAdapter(this);
		dbAdapter.open();
		dbAdapter.close();
	}

	/**
	 * Creates the menu.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		boolean result = super.onCreateOptionsMenu(menu);
		menu.add(0, RESET_ID, 0, R.string.menuReset);
		return result;
	}

	/**
	 * Shows the right dialog according to the menu item selected.
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case RESET_ID:
			showDialog(RESET_DIALOG);
			return true;
		}

		return super.onOptionsItemSelected(item);
	}

	/**
	 * Resets the game state.
	 */
	public void resetGameState() {
		currentTaskId = route.getFirstTaskId();
		route = new Route(dbAdapter, route.getRouteId());
		dbAdapter.open();
		dbAdapter.updateGameState(route.getRouteId(), currentTaskId);
		dbAdapter.close();
	}

	/**
	 * Creates the menu dialog.
	 */
	protected Dialog onCreateDialog(int id) {
		Dialog dialog;
		switch (id) {
		case RESET_DIALOG:
			// Creates the dialog.
			dialog = new Dialog(this);

			// Creates the linear layout.
			LinearLayout layout = new LinearLayout(this);
			layout.setOrientation(LinearLayout.VERTICAL);
			layout.setLayoutParams(new LayoutParams(50,
					LayoutParams.WRAP_CONTENT));

			// Creates the innerLowerLayout that holds the buttons.
			LinearLayout innerLowerLayout = new LinearLayout(this);
			innerLowerLayout.setOrientation(LinearLayout.HORIZONTAL);
			innerLowerLayout.setLayoutParams(new LayoutParams(
					LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
			innerLowerLayout.setPadding(1, 5, 1, 1);

			// Creates the password text.
			TextView text = new TextView(this);
			text.setText(getString(R.string.passwordRequired));

			// Creates the password field.
			final EditText inputText = new EditText(this);
			inputText.setSingleLine();
			inputText.setTransformationMethod(PasswordTransformationMethod
					.getInstance());

			// Creates the yesButton.
			Button yesButton = new Button(this);
			yesButton.setText(R.string.yes);
			yesButton.setLayoutParams(new LayoutParams(
					LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1.0f));
			yesButton.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {

					// If the text inputed matches the password, the database is
					// reset.
					if (inputText.getText().toString().equals(PASSWORD)) {
						dbAdapter.resetDatabase();
						resetGameState();
						dismissDialog(RESET_DIALOG);
					}
				}
			});

			// Creates the noButton.
			Button noButton = new Button(this);
			noButton.setText(R.string.no);
			noButton.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
					LayoutParams.WRAP_CONTENT, 1.0f));
			noButton.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {
					// Dismisses the dialog.
					dismissDialog(RESET_DIALOG);
				}
			});

			// Adds the buttons to the innerLowerLayout.
			innerLowerLayout.addView(yesButton);
			innerLowerLayout.addView(noButton);

			// Adds the views to the layout.
			layout.addView(text);
			layout.addView(inputText);
			layout.addView(innerLowerLayout);

			// Sets the title of the dialog.
			dialog.setTitle(getString(R.string.menuReset));

			dialog.setContentView(layout);

			return dialog;
		default:
			dialog = null;
		}
		return dialog;
	}

	/**
	 * Creates a welcome screen that is shown when entering the game for the
	 * first time.
	 */
	private void createWelcomeScreen() {
		// Creates a linear layout.
		LinearLayout layout = new LinearLayout(this);
		layout.setOrientation(LinearLayout.VERTICAL);

		// Adds a text field with the task description.
		TextView title = new TextView(this);
		title.setText("THE AMAZING CITY GAME");
		layout.addView(title);

		progress = new TextView(this);
		layout.addView(progress);
		
		penalties = new TextView(this);
		layout.addView(penalties);

		// Adds a button to go to next task.
		Button button = new Button(this);
		button.setText("NEXT TASK");
		button.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// Loads current Task.
				loadTask();
			}
		});
		layout.addView(button);

		// Sets the layout.
		setContentView(layout);
	}

	/**
	 * Loads the UI for the current task.
	 */
	private void loadTask() {
		Intent i = new Intent(this, TaskActivity.class);
		i.putExtra("currentTaskId", currentTaskId);
		i.putExtra("routeId", route.getRouteId());
		startActivityForResult(i, 0);
	}

	/**
	 * Loads the UI for the next task.
	 */
	private void loadNextTask() {
		currentTaskId = route.getNextTaskId(currentTaskId);
		dbAdapter.open();
		Log.e("Database", "Lagrer routeId = " + route.getRouteId());
		Log.e("Database", "Lagrer currentTaskId = " + currentTaskId);
		dbAdapter.updateGameState(route.getRouteId(), currentTaskId);
		dbAdapter.close();
		loadTask();
	}

	/**
	 * Checks that the route in the database corresponds with the route-object
	 * in this class. If changed, load the new route from the database.
	 */
	private void refreshRoute() {
		dbAdapter.open();
		int id = dbAdapter.getGameStateRoute();
		dbAdapter.close();
		if (id != route.getRouteId()) {
			route = new Route(dbAdapter, id);
		}
	}

	/**
	 * Called when an activity has been finished.
	 * 
	 * In this application this typically is when an user has finished a task
	 * and shall get a new one. The user will then be presented with a new task.
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		Bundle b = data.getExtras();
		int back = b.getInt("back");

		if (back == 0) {
			refreshRoute();
			loadNextTask();
		}

	}
	
	@Override
	protected void onResume() {
		super.onResume();
		dbAdapter.open();
		penalties.setText(getString(R.string.accumulatedPenalties)+" "+dbAdapter.getPenalties()+" minutes.");
		int[] routeIds = dbAdapter.getRoute(route.getRouteId());
		dbAdapter.close();
		int current = 0;
		for (int i=0; i<routeIds.length; i++) {
			if (routeIds[i] == currentTaskId) current = i+1;
		}
		if (current == 0) current = routeIds.length+1;
		progress.setText("Completed "+(current-1)+" of "+routeIds.length+" tasks.");
	}
	
	
}