package acg.prototype;

import acg.prototype.content.Task;
import acg.prototype.db.DbAdapter;
import acg.prototype.ui.BarcodeScanningTaskUI;
import acg.prototype.ui.CheckBoxTaskUI;
import acg.prototype.ui.GPSTaskUI;
import acg.prototype.ui.MultipleChoiceTaskUI;
import acg.prototype.ui.OpenTaskUI;
import acg.prototype.ui.ShazamTaskUI;
import acg.prototype.ui.TaskUI;
import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.text.method.PasswordTransformationMethod;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.TextView;

/**
 * The activity handling the UI of a given task.
 */
public class TaskActivity extends Activity {

	/**
	 * The request code used when the barcode scanner activity is invoked.
	 */
	public static final int BARCODE_SCANNER_REQUEST_CODE = 0x0ba7c0de;

	/**
	 * The taskUI for the current task;
	 */
	private TaskUI taskUI;

	/**
	 * The task the user is trying to solve.
	 */
	private Task task;
	
	/**
	 * Array with task IDs representing the route.
	 */
	private int[] routeIds;
	
	/**
	 * The dbAdapter for connecting to the database.
	 */
	private DbAdapter da;

	private static final int COMPLETE_ID = Menu.FIRST;
	private static final int COMPLETE_DIALOG = 1;
	
	/**
	 * Called when the activity is created. Fetches the current task (which is
	 * the same as the next task if a previous task was just solved, determines
	 * the task type, lays out the UI and awaits the user solving the task.
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		da = new DbAdapter(this);
		fetchTask();
		setContentView(fetchView());
	}

	/**
	 * Fetches the current task the user should solve from the database.
	 * 
	 * @return the current task from the database.
	 */
	private void fetchTask() {
		Bundle b = getIntent().getExtras();
		int currentTaskId = b.getInt("currentTaskId");
		da.open();
		task = da.getTask(currentTaskId);
		routeIds = da.getRoute(b.getInt("routeId"));
		da.close();
	}

	/**
	 * Creates the UI of the given task.
	 * 
	 * @return the UI of the given task.
	 */
	private View fetchView() {
		if(task == null){
			Log.e("UI", "no more tasks. Fetching finish screen.");
			Final finish = new Final(this);
			View v = finish.createView();
			return v;
		}
		switch (task.getTaskType()) {
		case Task.MULTIPLE_CHOICE_TASK:
			taskUI = new MultipleChoiceTaskUI(task, routeIds, this);
			break;
		case Task.BARCODE_SCANNING_TASK:
			taskUI = new BarcodeScanningTaskUI(task, routeIds, this);
			break;
		case Task.OPEN_TASK:
			taskUI = new OpenTaskUI(task, routeIds, this);
			break;
		case Task.GPS_TASK:
			taskUI = new GPSTaskUI(task, routeIds, this);
			break;
		case Task.SHAZAM_TASK:
			taskUI = new ShazamTaskUI(task, routeIds, this);
			break;
		case Task.CHECKBOX_TASK:
			taskUI = new CheckBoxTaskUI(task, routeIds, this);
			break;
		default:
			break;
		}
		return taskUI.createView();
	}

	/**
	 * Called if the task UI has started another activity/program and returned.
	 */
	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		if (requestCode == BARCODE_SCANNER_REQUEST_CODE) {
			if (resultCode == RESULT_OK) {
				String contents = intent.getStringExtra("SCAN_RESULT");
				BarcodeScanningTaskUI ui = (BarcodeScanningTaskUI) taskUI;
				ui.provideRoute(contents);
			}
		}
	}
	
	/**
	 * Creates the menu.
	 */
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
        boolean result = super.onCreateOptionsMenu(menu);
        menu.add(0, COMPLETE_ID, 0, R.string.menuComplete);
	    return result;
	}
	
	/**
	 * Shows the right dialog according to the menu item selected.
	 */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case COMPLETE_ID:
            showDialog(COMPLETE_DIALOG);
            return true;
        }
       
        return super.onOptionsItemSelected(item);
    }
  
 
    /**
     * Creates the menu dialog.
     */
    protected Dialog onCreateDialog(int id) {
        Dialog dialog;
        switch(id) {
        case COMPLETE_DIALOG:
        	// Creates the dialog.
        	dialog = new Dialog(this);
        	
        	// Creates the linear layout.
        	LinearLayout layout = new LinearLayout(this); 	
        	layout.setOrientation(LinearLayout.VERTICAL);
        	layout.setLayoutParams(new LayoutParams(50, LayoutParams.WRAP_CONTENT));
        
        	// Creates the innerLowerLayout that holds the buttons.
        	LinearLayout innerLowerLayout = new LinearLayout(this);
    		innerLowerLayout.setOrientation(LinearLayout.HORIZONTAL);
    		innerLowerLayout.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
    		innerLowerLayout.setPadding(1, 5, 1, 1);
        	
        	// Creates the password text.
        	TextView text = new TextView(this);
        	text.setText(getString(R.string.passwordRequired));
        	
        	// Creates the password field.
        	final EditText inputText = new EditText(this);
    		inputText.setSingleLine();
    		inputText.setTransformationMethod(PasswordTransformationMethod.getInstance());
    		
    		// Creates the yesButton.
    		Button yesButton = new Button(this);
    		yesButton.setText(R.string.yes);
    		yesButton.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1.0f));
    		yesButton.setOnClickListener(new OnClickListener() {
    			public void onClick(View v) {
    				// If the text inputed matches the password.
    				// The task is completed.
    				// Wrong answers and hints are reset so that user does not get any penalty.
    				if(inputText.getText().toString().equals(task.getBypassCode())){
    					da.open();    					
    					da.completeTask(task.getId());
    					da.close();
    					Intent i = new Intent();
    					i.putExtra("back", 0);
    					setResult(Activity.RESULT_OK, i);
    					finish();
    					dismissDialog(COMPLETE_DIALOG);
    				}
    			}
    		});
    		
    		// Creates the noButton.
    		Button noButton = new Button(this);
    		noButton.setText(R.string.no);
    		noButton.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1.0f));
    		noButton.setOnClickListener(new OnClickListener() {
    			public void onClick(View v) {
    				// Dismisses the dialog.
    				dismissDialog(COMPLETE_DIALOG);
    			}
    		});
    		
    		// Adds the buttons to the innerLowerLayout.
    		innerLowerLayout.addView(yesButton);
    		innerLowerLayout.addView(noButton);

    		// Adds the views to the layout.
    		layout.addView(text);
       		layout.addView(inputText);
    		layout.addView(innerLowerLayout);

    		// Sets the title of the dialog.
    		dialog.setTitle(getString(R.string.menuComplete));
        	dialog.setContentView(layout);
            
        	return dialog;
        default:
            dialog = null;
        }
        return dialog;
    }
    
	
	/**
	 * Overrides the default implementation of back button.
	 */
	public boolean onKeyDown(int keyCode, KeyEvent event) {
	     if (keyCode == KeyEvent.KEYCODE_BACK) {

	     //preventing default implementation previous to android.os.Build.VERSION_CODES.ECLAIR

			Intent i = new Intent();
			i.putExtra("back", 1);
			setResult(Activity.RESULT_OK, i);
			finish();
	     return true;

	     }

	     return super.onKeyDown(keyCode, event);    
	}
}