package acg.prototype.content;

import acg.prototype.db.DbAdapter;


/**
 * A task in the game.
 */
public abstract class Task {

	/**
	 * Denotes that the task is a multiple choice question.
	 */
	public static final int MULTIPLE_CHOICE_TASK = 0;

	/**
	 * Denotes that the task consists of finding and scanning a barcode.
	 */
	public static final int BARCODE_SCANNING_TASK = 1;
	
	/**
	 * Denotes that the task is an open answer task.
	 */
	public static final int OPEN_TASK = 2;
	
	/**
	 * Denotes that the task is a GPS task.
	 */
	public static final int GPS_TASK = 3;
	
	/**
	 * Denotes that the task is a shazam task.
	 */
	public static final int SHAZAM_TASK = 4;
	
	/**
	 * Denotes that the task is a checkbox task.
	 */
	public static final int CHECKBOX_TASK = 5;

	/**
	 * The id of the task.
	 */
	protected int id;

	/**
	 * A constant denoting which type of task this task is.
	 */
	protected int taskType;

	/**
	 * The description of the task.
	 */
	protected String description;	
	
	/**
	 * The questions for the this task.
	 */
	protected String[] questions;

	/**
	 * The time penalty in minutes for wrong answers.
	 */
	protected int penalty;

	/**
	 * The number of times the user has provided a wrong answer to the task.
	 */
	protected int wrongAnswers = 0;
	
	/**
	 * The database imageId for this task, 0 if the task does not have an image.
	 */
	protected int imageId;
	
	/**
	 * A boolean which denotes if the task is completed by the user or not.
	 */
	protected boolean completed;
	
	/**
	 * The list of hints belonging to this task.
	 */
	protected Hint[] hints;
	
	/**
	 * The bypass/failsafe code that is required to auto-complete this task, reset it's hints used and wrong answers given
	 */
	protected String bypassCode;
	
	/**
	 * A dbAdapter for connecting to the database.
	 */
	protected DbAdapter dbAdapter;

	/**
	 * Constructs a new task.
	 * 
	 * @param id - the database-id of the task.
	 * @param taskType - the task type of the task.
	 * @param description - the task description. 
	 * @param questions - the task questions.
	 * @param penalty - the penalty for wrong answers.
	 * @param imageId - the database id for the image for this task.
	 * @param hints - the hints for this task.
	 * @param completed - denotes if the task is completed or not.
	 * @param wrongAnswers - the number of times the user has provided a wrong answer to the task.
	 * @param da - DbAdapter for connecting to the database.
	 */
	
	public Task(int id, int taskType, String description, String[] questions, int penalty, int imageId, Hint[] hints, boolean completed, int wrongAnswers, String bypassCode, DbAdapter dbAdapter){
		this.id = id;
		this.taskType = taskType;
		this.description = description;
		this.questions = questions;
		this.penalty = penalty;
		this.imageId = imageId;
		this.hints = hints;
		this.completed = completed;
		this.wrongAnswers = wrongAnswers;
		this.bypassCode = bypassCode;
		this.dbAdapter = dbAdapter;
	}

	/**
	 * Returns the id of the task.
	 * 
	 * @return the id of the task.
	 */
	public int getId() {
		return id;
	}

	/**
	 * Returns a constant denoting which type of task this task is.
	 * 
	 * @return a constant denoting which type of task this task is.
	 */
	public int getTaskType() {
		return taskType;
	}

	/**
	 * Returns the description of the task.
	 * 
	 * @return the description of the task.
	 */
	public String getDescription() {
		return description;
	}
	
	/**
	 * Returns the time penalty for wrong answers.
	 * 
	 * @return the time penalty for wrong answers.
	 */
	public int getPenalty() {
		return penalty;
	}
	
	/**
	 * Returns the number of times the user has provided a wrong answer to the task.
	 * 
	 * @return the number of times the user has provided a wrong answer to the task.
	 */
	public int getWrongAnswers() {
		return wrongAnswers;
	}
	
	/**
	 * Increases the number of answers with one each time the user answers wrong.
	 */
	public void incWrongAnswers() {
		wrongAnswers++;
		dbAdapter.open();
		dbAdapter.updateTask(id, wrongAnswers, completed);
		dbAdapter.close();
	}
	
	/**
	 * Returns the R.java-reference of the image of the task.
	 * 
	 * @return the R.java-reference of the image of the task.
	 */
	
	public int getImageId() {
		return imageId;
	}
	
	
	/**
	 * Set the boolean which denotes if the task is completed or not.
	 * 
	 * @param bool - true if the task is completed, false if its not.
	 */
	public void setCompleted(boolean bool){
		this.completed = bool;
		dbAdapter.open();
		dbAdapter.updateTask(id, wrongAnswers, completed);
		dbAdapter.close();
	}
	
	
	/**
	 * Returns true if the task is completed, false if not.
	 * 
	 * @return true if the task is completed, false if not.
	 */
	public boolean getCompleted(){
		return completed;
	}
	
	/**
	 * Gets the remaining number of unused hints.
	 * @return - an int representing number of remaining unused hints
	 */
	public int getRemainingHints(){
		int counter = 0;
		for ( int i = 0; i<hints.length; i++ ) {
			if (!hints[i].isUsed()) counter++;
		}
		return counter;
	}
	
	/**
	 * Retrieves the next unused hint.
	 * @return the next unused hint
	 */
	public Hint getNextUnusedHint(){
		if (hints == null) {
			return null;
		}
		for ( int i=0; i<hints.length; i++) {
			if ( !hints[i].isUsed()) return hints[i];
		}
		return null;
	}
	
	/**
	 * Retrieves hints that have been used for this task.
	 * @return array of hints that have been used. Empty array if none are found.
	 */
	public Hint[] getUsedHints(){
		if (hints==null) return null;
		int counter = 0;
		for ( int i = 0; i<hints.length; i++ ) {
			if (hints[i].isUsed()) counter++;
			else break;
		}
		Hint[] result = new Hint[counter];
		for (int i=0; i<counter; i++) result[i] = hints[i];
		return result;
	}
	
	/**
	 * Sets the used state for the hint in the application and database to true.
	 * @param hint - the hint to be set to used
	 */
	public void setHintUsed(Hint hint) {
		for (int i = 0; i<hints.length; i++) {
			if (hints[i].getId() == hint.getId()) {
				hints[i].setUsed();
				dbAdapter.setHintUsed(hint.getId());
				break;
			}
		}
	}
	
	/**
	 * Validates the given answer by the user against the real answer stored in the task. 
	 * @param givenAnswer - The answer given by the user
	 * @return true if correct, false otherwise
	 */
	public abstract boolean validateTask(Object givenAnswer);
	
	/**
	 * Checks whether or not the task has hints.
	 * @return true if the task has hints, false otherwise
	 */
	public boolean hasHints() {
		if (hints==null || hints.length==0) return false;
		return true;
	}
	
	/**
	 * Gets the questions for this task
	 * @return string array with questions
	 */
	public String[] getQuestions() {
		return this.questions;
	}
	
	public String getBypassCode() {
		return this.bypassCode;
	}

}