package acg.prototype.db;

import acg.prototype.R;
import acg.prototype.content.Barcode;
import acg.prototype.content.Checkbox;
import acg.prototype.content.GPS;
import acg.prototype.content.Hint;
import acg.prototype.content.MultipleChoice;
import acg.prototype.content.Open;
import acg.prototype.content.Shazam;
import acg.prototype.content.Task;
import acg.prototype.util.GPSHandler;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;

/**
 * An adapter for communicating with the application's database.
 */
public class DbAdapter {

	/**
	 * The current activity.
	 */
	private Context context;

	/**
	 * The database connection helper.
	 */
	private DbHelper dbHelper;


	/**
	 * The database object to run queries on.
	 */
	private SQLiteDatabase db = null;

	/**
	 * Constructs a new database adapter.
	 * 
	 * @param context
	 *            - the current activity.
	 */
	public DbAdapter(Context context) {
		this.context = context;
		dbHelper = new DbHelper(context);
	}

	/**
	 * Opens the database, i.e. connects to it.
	 * 
	 * @return this (self reference, allowing this to be chained in an
	 *         initialization call).
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public DbAdapter open() throws SQLException {
		db = dbHelper.getWritableDatabase();
		return this;
	}

	public void resetDatabase() {
		open();
		dbHelper.resetDatabase(db);
		close();
	}

	/**
	 * Closes the connection to the database.
	 */
	public void close() {
		dbHelper.close();
	}

	/**
	 * Completes the task with id taskId and reset hints and wrong answers.
	 * 
	 * @param taskId
	 *            - the id of the task to update.
	 * @throws SQLException
	 */
	public void completeTask(int taskId) throws SQLException {
		ContentValues cv = new ContentValues();
		cv.put(context.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(context.getString(R.string.dbTaskCompleted), 1);
		db.update(context.getString(R.string.dbTask), cv,
				context.getString(R.string.dbTaskId) + "=" + taskId, null);

		cv.clear();
		cv.put(context.getString(R.string.dbHintUsed), 0);
		db.update(context.getString(R.string.dbHint), cv,
				context.getString(R.string.dbHintTaskId) + "=" + taskId, null);

	}

	/**
	 * Retrieves the task with the given id from the database.
	 * 
	 * @param taskId
	 *            - the id of the task to retrieve.
	 * @return the task with the given id retrieved from the database.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public Task getTask(int taskId) throws SQLException {
		String[] rowNames = { context.getString(R.string.dbTaskId),
				context.getString(R.string.dbTaskDescription),
				context.getString(R.string.dbTaskCompleted),
				context.getString(R.string.dbTaskImage),
				context.getString(R.string.dbTaskSolution),
				context.getString(R.string.dbTaskType),
				context.getString(R.string.dbTaskPenalty),
				context.getString(R.string.dbTaskWrongAnswers),
				context.getString(R.string.dbTaskBypassCode)};
		Cursor cursor = db.query(true, context.getString(R.string.dbTask),
				rowNames, context.getString(R.string.dbTaskId) + "=" + taskId,
				null, null, null, null, null);
		cursor.moveToFirst();

		// returns null if the task has a id that is not of any task in the
		// database
		// this basicly means that the game is finished
		if (cursor.getCount() == 0)
			return null;

		// The fields retrieved from the database
		String description = cursor.getString(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskDescription)));
		boolean completed = cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskDescription))) > 0;
		int imageId = cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskImage)));
		String answer = cursor.getString(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskSolution)));
		int type = cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskType)));
		int penalty = cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskPenalty)));
		int wrongAnswers = cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbTaskWrongAnswers)));
		String bypassCode = cursor.getString(cursor.getColumnIndexOrThrow(context.getString(R.string.dbTaskBypassCode)));

		Task t = null;

		switch (type) {

		case Task.OPEN_TASK:
			t = new Open(taskId, description, getQuestions(taskId), penalty, imageId,
					getHints(taskId), answer, completed,
					wrongAnswers, bypassCode, this);
			break;

		case Task.BARCODE_SCANNING_TASK:
			t = new Barcode(taskId, description, penalty, imageId,
					getHints(taskId), answer, completed, 
					wrongAnswers, bypassCode, this);
			break;

		case Task.MULTIPLE_CHOICE_TASK:
			t = new MultipleChoice(taskId, description, getQuestions(taskId),
					penalty, imageId, getHints(taskId),
					getAlternatives(taskId), answer, completed, wrongAnswers, bypassCode,
					this);
			break;

		case Task.CHECKBOX_TASK:
			String[] rowNamesCB = { context
					.getString(R.string.dbAlternativeAlternative) };
			cursor = db.query(
					true,
					context.getString(R.string.dbAlternative),
					rowNamesCB,
					context.getString(R.string.dbAlternativeIsCorrect)
							+ " = 1 AND "
							+ context.getString(R.string.dbAlternativeTaskId) + "="
							+ taskId, null, null, null, null, null);
			cursor.moveToFirst();

			String[] answers = new String[cursor.getCount()];
			for (int i = 0; i < cursor.getCount(); i++) {
				answers[i] = cursor.getString(cursor.getColumnIndexOrThrow(context
						.getString(R.string.dbAlternativeAlternative)));
				cursor.moveToNext();
			}
			t = new Checkbox(taskId, description, getQuestions(taskId), penalty,
					imageId, getHints(taskId), getAlternatives(taskId),
					answers, completed, wrongAnswers, bypassCode, this);
			break;

		case Task.GPS_TASK:
			String[] rowNamesGPS = { context.getString(R.string.dbGPSLatitude),
					context.getString(R.string.dbGPSLongitude),
					context.getString(R.string.dbGPSRadius) };
			cursor = db.query(true, context.getString(R.string.dbGPS), rowNamesGPS,
					context.getString(R.string.dbGPSTaskId) + "=" + taskId, null,
					null, null, null, null);
			cursor.moveToFirst();

			String longitude = cursor.getString(cursor
					.getColumnIndexOrThrow(context
							.getString(R.string.dbGPSLongitude)));
			String latitude = cursor.getString(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbGPSLatitude)));
			int radius = cursor.getInt(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbGPSRadius)));

			t = new GPS(taskId, description, penalty, imageId,
					getHints(taskId), completed, wrongAnswers, bypassCode, this,
					new GPSHandler(context), Double.parseDouble(latitude),
					Double.parseDouble(longitude), radius);
			break;

		case Task.SHAZAM_TASK:
			String[] rowNamesShazam = { context.getString(R.string.dbShazamSong) };
			cursor = db.query(true, context.getString(R.string.dbShazam),
					rowNamesShazam, context.getString(R.string.dbShazamTaskId)
							+ "=" + taskId, null, null, null, null, null);
			cursor.moveToFirst();

			int songId = cursor.getInt(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbShazamSong)));
			t = new Shazam(taskId, description, getQuestions(taskId), penalty, imageId,
					getHints(taskId), answer, completed, 
					wrongAnswers, songId, bypassCode, this);
			break;

		default:
			break;

		}

		return t;
	}

	/**
	 * Retrieves number of tasks in Task table.
	 * 
	 * @return number of tasks in Task table.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public int getNumberOfTasks(int routeId) throws SQLException {
		String[] rowNames = { context.getString(R.string.dbRouteElementId),
				context.getString(R.string.dbRouteElementRouteId) };
		Cursor cursor = db.query(true, context.getString(R.string.dbRouteElement),
				rowNames, context.getString(R.string.dbRouteElementRouteId) + " = "
						+ routeId, null, null, null, null, null);
		return cursor.getCount();
	}

	/**
	 * Retrieves number of routes in RouteElement table.
	 * 
	 * @return number of routes in RouteElement table.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public int getNumberOfRoutes() throws SQLException {
		String[] rowNames = { context.getString(R.string.dbRouteElementRouteId) };
		Cursor cursor = db.query(true, context.getString(R.string.dbRouteElement),
				rowNames, null, null, null, null, null, null);
		return cursor.getCount();
	}

	/**
	 * Updates the task in the database.
	 * 
	 * @param taskId
	 *            - the task id.
	 * @param wrongAnswers
	 *            - an updated number of wrong answers.
	 * @param completed
	 *            - denotes if the task is completed or not.
	 * @throws SQLException
	 *             - if the database could not be opened or created.
	 */

	public void updateTask(int taskId, int wrongAnswers, boolean completed)
			throws SQLException {
		ContentValues cv = new ContentValues();
		cv.put(context.getString(R.string.dbTaskWrongAnswers), wrongAnswers);
		if (completed) {
			cv.put(context.getString(R.string.dbTaskCompleted), 1);
		} else {
			cv.put(context.getString(R.string.dbTaskCompleted), 0);
		}

		db.update(context.getString(R.string.dbTask), cv,
				context.getString(R.string.dbTaskId) + "=" + taskId, null);

	}

	/**
	 * Retrieves a route from the database.
	 * 
	 * @param routeId
	 *            - the id of the route to be retrieved.
	 * @return The route with id == routeId.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public int[] getRoute(int routeId) throws SQLException {
		int[] result = new int[getNumberOfTasks(routeId)];

		String[] rowNames = { context.getString(R.string.dbRouteElementTaskId) };
		Cursor cursor = db.query(false, context.getString(R.string.dbRouteElement),
				rowNames, context.getString(R.string.dbRouteElementRouteId) + "="
						+ routeId, null, null, null,
				context.getString(R.string.dbRouteElementId), null);
		cursor.moveToFirst();

		for (int i = 0; i < cursor.getCount(); i++) {
			result[i] = (cursor.getInt(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbRouteElementTaskId))));
			cursor.moveToNext();
		}

		return result;
	}

	/**
	 * Retrieves the current route from the GameState table.
	 * 
	 * @return - the current route.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public int getGameStateRoute() throws SQLException {
		String[] rowNames = { context.getString(R.string.dbGameStateRouteId) };
		Cursor cursor = db.query(true, context.getString(R.string.dbGameState),
				rowNames, context.getString(R.string.dbGameStateId) + "=" + 1,
				null, null, null, null, null);
		cursor.moveToFirst();

		return cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbGameStateRouteId)));
	}

	/**
	 * Retrieves the current task from the GameState table.
	 * 
	 * @return - the current task.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public int getGameStateTaskId() throws SQLException {
		String[] rowNames = { context.getString(R.string.dbGameStateCurrentTaskId) };
		Cursor cursor = db.query(true, context.getString(R.string.dbGameState),
				rowNames, context.getString(R.string.dbGameStateId) + "=" + 1,
				null, null, null, null, null);
		cursor.moveToFirst();

		return cursor.getInt(cursor.getColumnIndexOrThrow(context
				.getString(R.string.dbGameStateCurrentTaskId)));
	}

	/**
	 * Updates the GameState table.
	 * 
	 * @param routeId
	 *            - the current route.
	 * @param currentTaskId
	 *            - the current task.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public void updateGameState(int routeId, int currentTaskId)
			throws SQLException {
		ContentValues cv = new ContentValues();
		cv.put(context.getString(R.string.dbGameStateCurrentTaskId), currentTaskId);
		cv.put(context.getString(R.string.dbGameStateRouteId), routeId);
		db.update(context.getString(R.string.dbGameState), cv,
				context.getString(R.string.dbGameStateId) + "=" + 1, null);

	}

	/**
	 * Updates the GameState table.
	 * 
	 * @param routeId
	 *            - the current route.
	 * @throws SQLException
	 *             - if the database could be neither opened or created.
	 */
	public void updateGameState(int routeId) throws SQLException {
		ContentValues cv = new ContentValues();
		cv.put(context.getString(R.string.dbGameStateRouteId), routeId);
		db.update(context.getString(R.string.dbGameState), cv,
				context.getString(R.string.dbGameStateId) + "=" + 1, null);

	}

	/**
	 * Finds the total penalty accumulated so far. For each task: penalty *
	 * wrongAnswers. For hints: penalty if used
	 * 
	 * @return sum of all penalties incurred.
	 */
	public int getPenalties() {
		int result = 0;

		// get tasks and sum up penalties * wrong anwers given
		String[] rowNames = { context.getString(R.string.dbTaskId),
				context.getString(R.string.dbTaskWrongAnswers),
				context.getString(R.string.dbTaskPenalty) };
		Cursor cursor = db.query(true, context.getString(R.string.dbTask),
				rowNames, null, null, null, null,
				context.getString(R.string.dbTaskId), null);
		cursor.moveToFirst();

		for (int i = 0; i < cursor.getCount(); i++) {
			result += cursor.getInt(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbTaskPenalty)))
					* cursor.getInt(cursor.getColumnIndexOrThrow(context
							.getString(R.string.dbTaskWrongAnswers)));
			cursor.moveToNext();
		}

		// get all used hints and sum up penalties
		String[] rowNames2 = { context.getString(R.string.dbHintId),
				context.getString(R.string.dbHintPenalty),
				context.getString(R.string.dbHintTaskId) };
		Cursor cursor2 = db.query(true, context.getString(R.string.dbHint),
				rowNames2, context.getString(R.string.dbHintUsed) + "=" + 1, null,
				null, null, context.getString(R.string.dbHintId), null);
		if (cursor2.getCount() != 0) {
			cursor2.moveToFirst();
			for (int i = 0; i < cursor2.getCount(); i++) {
				result += cursor2.getInt(cursor2.getColumnIndexOrThrow(context
						.getString(R.string.dbHintPenalty)));
				cursor2.moveToNext();
			}
		}
		return result;
	}

	/**
	 * Retrieves all hints belonging to a given task
	 * 
	 * @param taskId
	 *            - id of the task
	 * @return array of hints belonging to a given task.
	 */
	public Hint[] getHints(int taskId) {

		String[] rownames = new String[] { context.getString(R.string.dbHintId),
				context.getString(R.string.dbHintDescription),
				context.getString(R.string.dbHintTaskId),
				context.getString(R.string.dbHintPenalty),
				context.getString(R.string.dbHintUsed),
				context.getString(R.string.dbHintImage),
				context.getString(R.string.dbHintSong), };
		Cursor cursor = db.query(true, context.getString(R.string.dbHint),
				rownames, context.getString(R.string.dbHintTaskId) + "=" + taskId,
				null, null, null, context.getString(R.string.dbHintId), // order by
				null);

		Hint[] hints = new Hint[cursor.getCount()];
		if (cursor.getCount() != 0) {
			cursor.moveToFirst();
			for (int i = 0; i < cursor.getCount(); i++) {
				hints[i] = new Hint(
						cursor.getInt(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintId))),
						cursor.getString(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintDescription))),
						cursor.getInt(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintPenalty))),
						cursor.getInt(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintUsed))) != 0,
						cursor.getInt(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintImage))),
						cursor.getInt(cursor.getColumnIndexOrThrow(context
								.getString(R.string.dbHintSong))));
				cursor.moveToNext();
			}
		}
		return hints;

	}

	/**
	 * Sets the state of the hint to used
	 * 
	 * @param hintId
	 *            - id of the target hint
	 */
	public void setHintUsed(int hintId) {
		open();
		ContentValues cv = new ContentValues();
		cv.put(context.getString(R.string.dbHintUsed), 1);
		db.update(context.getString(R.string.dbHint), cv,
				context.getString(R.string.dbHintId) + "=" + hintId, null);
		close();
	}

	/**
	 * Gets the questions belonging to a task.
	 * 
	 * @param taskId
	 *            - id of the given task
	 * @return array of strings
	 */
	public String[] getQuestions(int taskId) {
		String[] rowNames = { context.getString(R.string.dbQuestionQuestion) };
		Cursor cursor = db.query(true, context.getString(R.string.dbQuestion),
				rowNames, context.getString(R.string.dbQuestionTaskId) + "="
						+ taskId, null, null, null, context.getString(R.string.dbQuestionId), null);

		String[] questions = new String[cursor.getCount()];
		if (cursor.getCount() != 0) {
			cursor.moveToFirst();
			for (int i = 0; i < cursor.getCount(); i++) {
				questions[i] = (cursor.getString(cursor
						.getColumnIndexOrThrow(context
								.getString(R.string.dbQuestionQuestion))));
				cursor.moveToNext();
			}
		}
		return questions;
	}

	/**
	 * Gets the answer alternatives for a task. Applicable for multiple choice and checkbox tasks.
	 * @param taskid
	 * 				- id of the task.
	 * @return array of strings with alternatives.
	 */
	public String[] getAlternatives(int taskid) {
		String[] rowNames = { context.getString(R.string.dbAlternativeAlternative) };
		Cursor cursor = db.query(true, context.getString(R.string.dbAlternative),
				rowNames, context.getString(R.string.dbAlternativeTaskId) + "="
						+ taskid, null, null, null, null, null);
		cursor.moveToFirst();

		String[] alternatives = new String[cursor.getCount()];

		for (int i = 0; i < cursor.getCount(); i++) {
			alternatives[i] = cursor.getString(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbAlternativeAlternative)));
			cursor.moveToNext();
		}
		return alternatives;
	}

	/**
	 * Returns the route of the given keyword, or <code>0</code> if it doesn't
	 * exist.
	 * 
	 * @param keyword
	 *            the keyword to find the route for.
	 * @return the route of the given keyword, or <code>0</code> if it doesn't
	 *         exist.
	 */
	public int getRoute(String keyword) {
		String[] rowNames = { context.getString(R.string.dbRouteId) };
		Cursor cursor = db.query(true, context.getString(R.string.dbRoute),
				rowNames, context.getString(R.string.dbRouteKeyword) + "=\""
						+ keyword + "\"", null, null, null, null, null);

		if (cursor.getCount() != 0) {
			cursor.moveToFirst();
			return cursor.getInt(cursor.getColumnIndexOrThrow(context
					.getString(R.string.dbRouteId)));
		}

		return 0;
	}
}