package acg.prototype.db;

import acg.prototype.R;
import acg.prototype.content.Task;
import android.content.ContentValues;
import android.content.Context;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * The class handling connections to the database.
 * 
 * This includes handling creation of and upgrades of the database.
 */
public class DbHelper extends SQLiteOpenHelper {

	/**
	 * The activity invoking this database helper.
	 */
	private Context ctx;

	/**
	 * Creates a new database helper.
	 * 
	 * @param context
	 *            the current activity.
	 */
	public DbHelper(Context context) {
		super(context, "acg", null, 3);
		this.ctx = context;
	}

	/**
	 * Creates the database.
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		Log.e("DB", "onCreate");

		// Creates the database tables
		createTables(db);

		// Fills in the tables with data
		fillDatabase(db);
	}

	/**
	 * Fills the database with game content.
	 */

	public void fillDatabase(SQLiteDatabase db) {

		// The ContentValues object used for every insertion.
		ContentValues cv = new ContentValues();

		// The start gamestate.
		createGameState(db, cv);

		// The tasks of the game.
		createTasks(db, cv);

		// The routes.
		createRoutes(db, cv);

	}

	/**
	 * Creates the database tables.
	 * 
	 * @param db
	 *            - the database.
	 */
	public void createTables(SQLiteDatabase db) {

		// Task table.
		String createTaskTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbTask) + " ("
				+ ctx.getString(R.string.dbTaskId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbTaskDescription) + " text, "
				+ ctx.getString(R.string.dbTaskCompleted) + " integer, "
				+ ctx.getString(R.string.dbTaskImage) + " integer, "
				+ ctx.getString(R.string.dbTaskSolution) + " text, "
				+ ctx.getString(R.string.dbTaskType) + " integer not null, "
				+ ctx.getString(R.string.dbTaskPenalty) + " integer not null, "
				+ ctx.getString(R.string.dbTaskWrongAnswers) + " integer, "
				+ ctx.getString(R.string.dbTaskBypassCode) + " text not null);";
		db.execSQL(createTaskTable);

		// RouteElement table.
		String createRouteElementTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbRouteElement) + " ("
				+ ctx.getString(R.string.dbRouteElementId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbRouteElementRouteId)
				+ " integer not null, "
				+ ctx.getString(R.string.dbRouteElementTaskId)
				+ " integer not null);";
		db.execSQL(createRouteElementTable);

		// RouteElement table.
		String createRouteTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbRoute) + " ("
				+ ctx.getString(R.string.dbRouteId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbRouteKeyword) + " text);";
		db.execSQL(createRouteTable);

		// GameState table.
		String createGameStateTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbGameState) + " ("
				+ ctx.getString(R.string.dbGameStateId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbGameStateRouteId)
				+ " integer not null, "
				+ ctx.getString(R.string.dbGameStateCurrentTaskId)
				+ " integer);";
		db.execSQL(createGameStateTable);

		// Hint table.
		String createHintTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbHint) + " ("
				+ ctx.getString(R.string.dbHintId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbHintDescription) + " text, "
				+ ctx.getString(R.string.dbHintTaskId) + " integer not null, "
				+ ctx.getString(R.string.dbHintPenalty) + " integer, "
				+ ctx.getString(R.string.dbHintUsed) + " integer, "
				+ ctx.getString(R.string.dbHintImage) + " integer, "
				+ ctx.getString(R.string.dbHintSong) + " integer);";
		db.execSQL(createHintTable);

		// Alternative table.
		String createAlternativeTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbAlternative) + " ("
				+ ctx.getString(R.string.dbAlternativeId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbAlternativeAlternative) + " text, "
				+ ctx.getString(R.string.dbAlternativeTaskId)
				+ " integer not null, "
				+ ctx.getString(R.string.dbAlternativeIsCorrect) + " integer);";
		db.execSQL(createAlternativeTable);

		// Question table.
		String createQuestionTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbQuestion) + " ("
				+ ctx.getString(R.string.dbQuestionId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbQuestionQuestion) + " text, "
				+ ctx.getString(R.string.dbQuestionTaskId)
				+ " integer not null);";
		db.execSQL(createQuestionTable);

		// GPS table.
		String createGPSTable = "CREATE TABLE " + ctx.getString(R.string.dbGPS)
				+ " (" + ctx.getString(R.string.dbGPSId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbGPSLongitude) + " text, "
				+ ctx.getString(R.string.dbGPSLatitude) + " text, "
				+ ctx.getString(R.string.dbGPSRadius) + " integer, "
				+ ctx.getString(R.string.dbGPSTaskId) + " integer not null);";
		db.execSQL(createGPSTable);

		// Shazam table.
		String createShazamTable = "CREATE TABLE "
				+ ctx.getString(R.string.dbShazam) + " ("
				+ ctx.getString(R.string.dbShazamId)
				+ " integer primary key autoincrement, "
				+ ctx.getString(R.string.dbShazamSong) + " integer, "
				+ ctx.getString(R.string.dbShazamTaskId)
				+ " integer not null);";
		db.execSQL(createShazamTable);
	}

	/**
	 * Creates a new game state.
	 * 
	 * @param db
	 *            - the database.
	 * @param cv
	 *            - the content values.
	 */
	public void createGameState(SQLiteDatabase db, ContentValues cv) {
		cv.clear();
		cv.put(ctx.getString(R.string.dbGameStateCurrentTaskId), 0);
		cv.put(ctx.getString(R.string.dbGameStateRouteId), 0);
		db.insert(ctx.getString(R.string.dbGameState), null, cv);
	}

	/**
	 * Creates the tasks.
	 * 
	 * @param db
	 *            - the database.
	 * @param cv
	 *            - the content values.
	 */
	public void createTasks(SQLiteDatabase db, ContentValues cv) {
		cv.clear();
		long taskId = -1;

		// ID1: Welcome
		taskId = createBarcodeTask(
				db,
				"Welcome to The Amazing City Game. Please scan a barcode in order to retrieve a game route.",
				0, "", 0, "hjerne");

		// ID2 : Kristiansten Fortress
		taskId = createCheckBoxTask(
				db,
				"Kristiansten Fortress is built on the strategically important height, Erlendshaug. "
						+ "There were initially plans made for a fortress at Erlendshaug already in 1676, but "
						+ "only after the great city fire in 1681 were the plans set in motion. The fortress "
						+ "had an important role to play in the city's fortification plan designed by Cicignon "
						+ "and Coucheron, but was decomissioned in 1816. WARNING! There is a long way down, watch your step! ", 0,
				4, "sverige");
		addQuestion(db, taskId,
				"Which of the following objects are at Kristiansten fortress?");
		addAlternative(db, taskId, "Cannons", true);
		addAlternative(db, taskId, "Gallows", false);
		addAlternative(db, taskId, "Barbed wire", true);
		addAlternative(db, taskId, "Memorial to fallen resistance fighters",
				true);
		addAlternative(db, taskId, "Moat", false);
		addHint(db, taskId, "There are no gallows", 0, 0, 2);
		addHint(db, taskId, "There is no moat", 0, 0, 2);

		// ID3: Kristiansten
		taskId = createOpenTask(
				db,
				"In this task we want you to find a famous building in Trondheim. "
						+ "This task will require some mathematics:\n"
						+ "X is the number of cannons standing on line next to the donjon.\n"
						+ "Y is the number of arched openings in the donjon.\n"
						+ "Z = ( X + Y ) * 8,4375.\n"
						+ "Stand on the brick wall above the western entrance "
						+ "(between the donjon and the cannons, right in front of some barbed wire) and face north. \n"
						+ "Then turn Z degrees.", 0, "Nidarosdomen", 4, "vinkel");
		addQuestion(db, taskId, "Which landmark are you looking directly at?");
		addHint(db, taskId, "Turn west", 0, 0, 2);
		
		// ID4 : Locating Nidarosdomen
		taskId = createGPSTask(db,
				"Travel to the place the following image from 1857 depicts.",
				3, 4, "63.42701614318537", "10.396628379821777", 100,
				"sjokolade");
		addHint(db, taskId, "Olav den Hellige is buried at the location.", 0,
				0, 2);
		addHint(db, taskId, "The location is Nidarosdomen.", 0, 0, 2);

		// ID5 : The war memorial
		String description = "On the north side of Nidaros Cathedral there is a war memorial. "
				+ "This particular war memorial was raised in the memory of the "
				+ "people that gave their lives for Norway during World War II.";
		taskId = createOpenTask(db, description, 0,
				"Til minne om frihetskampens ofre", 4, "kjakan");
		addQuestion(db, taskId,
				"This memorial carries an inscription. What does it say?");
		addHint(db, taskId, "The statue is close to a path.", 0, 0, 2);
		addHint(db, taskId, "It is a statue of a woman holding a child", 0, 0,
				2);

		// ID6 : The main facade
		description = "Nidaros Cathedral's Westfront is the cathedral's main facade. "
				+ "It went through a restoration period between 1901 and 1969. "
				+ "A large number of figures are placed in alcoves.";
		taskId = createMultipleChoiceTask(db, description, 0, "A scepter", 4,
				"paven");
		addQuestion(db, taskId,
				"The 8th figure from the right on the top row is carrying what item?");
		addAlternative(db, taskId, "A cross", false);
		addAlternative(db, taskId, "An axe", false);
		addAlternative(db, taskId, "A scepter", true);
		addAlternative(db, taskId, "A book", false);
		addHint(db, taskId, "The item is not a cross.", 0, 0, 2);
		addHint(db, taskId, "The item is not an axe.", 0, 0, 2);
		addHint(db, taskId, "The item is not a book.", 0, 0, 2);
		addHint(db, taskId, "The correct answer is a scepter!", 0, 0, 2);

		// ID7 : Locating Stiftsgrden
		taskId = createGPSTask(db,
				"Travel to the royal residence of the city.", 0, 4,
				"63.43151283743752", "10.394783020019531", 100, "rik");

		addHint(db, taskId, "It is the largest wooden building in the city.",
				0, 0, 5);
		addHint(db, taskId, "An image of the location.", 6, 0, 2);
		addHint(db, taskId, "The location is Stiftsgrden in Munkegata.", 0, 0,
				2);

		// ID8 : The iron gate
		description = "Stiftsgrden is possibly the largest wooden building in the Nordic countries. It is over 4000 square meters and has 140 "
				+ "rooms. Construction started in 1774 and it was originally designed as a private residence, but in 1906 it became the royal "
				+ "residence in Trondheim. Behind the building there is a park that is open to the public.";
		taskId = createOpenTask(db, description, 0, "1906", 4, "gruve");
		addQuestion(db, taskId, "In what year was the gate erected?");
		addHint(db, taskId,
				"The answer is written on the Iron Gate of the park.", 0, 0, 2);

		// ID9 : Locating Gamle Bybro
		taskId = createGPSTask(db, "Travel to the bridge that has portals.", 0,
				4, "63.42823036816747", "10.401585102081299", 50, "niding");
		addHint(db, taskId, "An image of the location", 5, 0, 2);
		addHint(db,
				taskId,
				"The destination is Gamle Bybro between Midtbyen and Bakklandet.",
				0, 0, 2);

		// ID10 : Bridge Building
		description = "The Old Town Bridge was originally built around 1681 as a result of Cicignons city plan. It was planned to cross "
				+ "the river further south, but to increase the defensive capabilities of the city it was built were it stands today. "
				+ "This location would give the city adequate defense if it was attacked from Vollan and vre Bakklandet. Later the "
				+ "bridge was rebuilt.";
		taskId = createOpenTask(db, description, 0, "1861", 4, "civ");
		addQuestion(db, taskId, "In what year was the bridge rebuilt?");
		addHint(db, taskId,
				"One possible source of information could be Wikipedia.", 0, 0,
				2);

		// ID11 : Lykkens Portal
		taskId = createOpenTask(db, "", 0, "Lykkens Portal", 4, "valve");
		addQuestion(db, taskId, "What is the name of the bridge portal?");
		addHint(db, taskId, "The answer contains two words.", 0, 0, 2);
		addHint(db, taskId, "\"Nidelven stille og vakker du er.\"", 0, 0, 2);

		// ID12 : Locating Britannia
		taskId = createGPSTask(
				db,
				"Travel to the place where the pike Jonathan was executed during World War 2.",
				0, 4, "63.43179115818105", "10.398699045181274", 50, "india");
		addHint(db, taskId, "An old image of the location.", 1, 0, 2);
		addHint(db, taskId, "The location is Britannia Hotel.", 0, 0, 2);

		// ID13 : The production company
		description = "Trondheim has been a cultural center for the Trndelag region for a long time. "
				+ "Part of this has been because of the city being a religious center, "
				+ "which can be seen and heard in the annual festival Olavsfestivalen that has a strong focus on church music. "
				+ "But the city has also fostered musical successes in the modern age, "
				+ "and in particular a duo that amongst other things is connected to the musical piece you can hear "
				+ "when pressing the Play button.";
		taskId = createShazamTask(db, description, 0, "Stargate", 4, 2,
				"rihanna");
		addQuestion(db, taskId, "What is the name of the duo?");
		String hint1 = "The song you just heard was Curious by Noora Noor "
				+ "(which can be checked by using the Android app Shazam), a Norwegian soul artist. "
				+ "But the duo has also worked with international artists, such as those behind this song.";
		String hint2 = "The song you heard was S Club 7 and their song S Club. "
				+ "The name of the duo is actually the name of their production company, "
				+ "as the duo works with writing and producing songs for other artists. "
				+ "S Club was one of their first hits abroad, but they have been even more successful than this, "
				+ "and have actually produced numerous number 1 hits in the USA. One of those hits are the following song.";
		addHint(db, taskId, hint1, 0, 3, 2);
		addHint(db, taskId, hint2, 0, 1, 2);

		// ID14 : The flag
		taskId = createOpenTask(
				db,
				"The following image depicts a specific flower. A large version of the image is on one of the "
						+ "sheets of paper you have brought with you.", 2,
				"Nyperose", 2, "rosenborg");
		addQuestion(db, taskId,
				"What is the Norwegian name of the flower depicted?");
		addHint(db, taskId,
				"You can use Google Goggles to determine which flag it is.", 0,
				0, 2);
		addHint(db, taskId, "The flower is a type of rose.", 0, 0, 2);
		addHint(db,
				taskId,
				"The fruit of the flower has a powder inside that is very itchy.",
				0, 0, 2);


		// ID15: Locating Vr Frue Kirke
		taskId = createGPSTask(
				db,
				"Travel to the building that Bjrn Sigvardsson is the architect of.",
				0, 4, "63.43018357866649", "10.39829134941101", 50, "foss");
		addHint(db, taskId, "It is a stone church.", 0, 0, 2);
		addHint(db, taskId, "It was built in the middle ages.", 0, 0, 2);
		addHint(db, taskId, "St. Mary.", 0, 0, 2);
		addHint(db, taskId, "An image of the location.", 8, 0, 2);
		addHint(db, taskId, "The location is Vr Frue Kirke.", 0, 0, 2);

		// ID16: Vr Frue Kirke Facts
		description = "Vr Frue Kirke is a stone church from the middle ages that has been rebuilt several times after burning down in the many city fires.";
		taskId = createOpenTask(db, description, 0, "Amsterdam", 4, "slitsom");
		addQuestion(db, taskId,
				"In which European capital were two of todays church clocks made?");
		addHint(db,
				taskId,
				"One of the curch clocks is located just inside the main entrance.",
				0, 0, 2);
		addHint(db, taskId, "The clocks was forged by Jan Albert de Grave.", 0,
				0, 2);
		addHint(db, taskId, "It is the capital of the Netherlands.", 0, 0, 2);

		// ID17: Locating Torvet
		taskId = createGPSTask(db,
				"Travel to the statue that Wilhelm Robert Rasmussen created.",
				0, 4, "63.43048110764693", "10.39505660533905", 50, "midten");
		addHint(db, taskId, "It is a statue of the founder of Trondheim.", 0,
				0, 2);
		addHint(db, taskId, "The statue is very tall.", 0, 0, 2);
		addHint(db, taskId, "An image of the location.", 7, 0, 2);
		addHint(db, taskId, "The location is torvet.", 0, 0, 2);

		// ID18: Torvet Shopsavvy
		description = "Trondheim has had a town square and marketplace since the early 12th century. After the great fire in 1681, the marketplace is situated in the heart of Trondheim, which corresponds to Cicignon's city plan. We want you to find the name of a prominent person in the city's history, by answering the following questions and following the instructions on paper for the ShopSavvy Challenge.";
		taskId = createOpenTask(db, description, 0, "OLAV", 4, "turbo");
		addQuestion(db, taskId,
				"How many points does the star on the ground have?");
		addQuestion(db, taskId, "When was the statue erected?");
		addQuestion(db, taskId, "When was Trondheim established?");
		addQuestion(db, taskId, "When did Olav Tryggvason die?");
		addHint(db, taskId, "For Q2: www.trondheim.com", 0, 0, 2);
		addHint(db, taskId, "For Q3: The founder is Olav Tryggvason.", 0, 0, 2);
		addHint(db, taskId, "For Q4: It is a round number.", 0, 0, 2);

		// ID19 : Locating Ravnkloa
		taskId = createGPSTask(db, "Travel to the fish market.", 0, 4,
				"63.43384968800512", "10.39306640625", 50, "lofoten");
		addHint(db, taskId, "An image of the location.", 4, 0, 2);
		addHint(db, taskId, "The destination is Ravnkloa in Munkegata.", 0, 0,
				2);

		// ID20 : The Writer
		description = "The statue at Ravnkloa depicts a man from the book \"The Last of the Vikings\" about local fishermen.";
		taskId = createMultipleChoiceTask(db, description, 0, "Johan Bojer", 4,
				"gris");
		addQuestion(db, taskId, "Who wrote that book?");
		addAlternative(db, taskId, "Olav Duun", false);
		addAlternative(db, taskId, "Henrik Ibsen", false);
		addAlternative(db, taskId, "Magnhild Haalke", false);
		addAlternative(db, taskId, "Johan Bojer", true);
		addHint(db, taskId, "The book was written in 1921.", 0, 0, 2);
		
		// ID21 : The Avenue
		description = "Munkegata is one of the very few Avenues in Norway.";
		taskId = createMultipleChoiceTask(db, description, 0, "37,65 meters",
				4, "fem");
		addQuestion(db, taskId, "How wide is Munkegata?");
		addAlternative(db, taskId, "30,00 meters", false);
		addAlternative(db, taskId, "33,33 meters", false);
		addAlternative(db, taskId, "37,65 meters", true);
		addAlternative(db, taskId, "40,00 meters", false);
		addHint(db,
				taskId,
				"When Cicignon made the city plan he made Munkegata '60 ell' wide.",
				0, 0, 2);
		addHint(db, taskId, "1 Norwegian ell in 1681 = 62,75 centimeters.", 0,
				0, 2);

		// ID22 : The End
		taskId = createOpenTask(
				db,
				"The last question of the race concerns Cicignons town plan. "
						+ "He made it so one of the streets, Munkegata, lies in the axis between Nidarosdomen and "
						+ "another famous landmark.", 0, "Munkholmen", 4,
				"slutt");
		addQuestion(db, taskId, "Which landmark?");
		addHint(db, taskId, "The landmark is an island.", 0, 0, 2);

	}

	/**
	 * Creates the routes.
	 * 
	 * @param db
	 *            - the database.
	 * @param cv
	 *            - the content values.
	 */
	public void createRoutes(SQLiteDatabase db, ContentValues cv) {

		/*
		Welcome = 1
		Kristiansten = 2, 3
		Nidarosdomen = 4, 5, 6
		Stiftsgrden = 7, 8
		Gamle Bybro = 9, 10, 11
		Britannia = 12, 13, 14
		Vr Frue Kirke = 15, 16
		Torvet = 17, 18
		Ravnkloa = 19, 20, 21, 22
		*/	
		
		// Route 1: Welcome - Kristiansten - Nidarosdomen - Britannia - Gamle Bybro - Vr Frue Kirke - Stiftsgrden - Torvet - Ravnkloa
		int[] elements1 = { 1, 
				2, 3, 
				4, 5, 6, 
				12, 13, 14, 
				9, 10, 11, 
				15, 16,
				7, 8, 
				17, 18, 
				19, 20, 21, 22 };
		
		// Route 2: Welcome - Kristiansten - Britannia - Vr Frue Kirke - Gamle Bybro - Nidarosdomen - Stiftsgrden - Torvet - Ravnkloa
		int[] elements2 = { 1, 
				2, 3, 
				12, 13, 14, 
				15, 16, 
				9, 10, 11, 
				4, 5, 6,
				7, 8, 
				17, 18, 
				19, 20, 21, 22 };
	
		// Route 3: Welcome - Kristiansten - Gamle Bybro - Britannia - Nidarosdomen - Stiftsgrden - Torvet - Vr Frue Kirke - Ravnkloa
		int[] elements3 = { 1, 
				2, 3, 
				9, 10, 11, 
				12, 13, 14, 
				4, 5, 6, 
				7, 8,
				17, 18,
				15, 16, 
				19, 20, 21, 22 };
		
		// Route 4: Welcome - Kristiansten - Vr Frue Kirke - Stiftsgrden - Nidarosdomen - Gamle Bybro - Britannia - Torvet - Ravnkloa
		int[] elements4 = { 1, 
				2, 3, 
				15, 16, 
				7, 8, 
				4, 5, 6, 
				9, 10, 11, 
				12, 13, 14, 
				17, 18, 
				19, 20, 21, 22 };

		createRoute(db, cv, 1, "alsjdfah", elements1);
		createRoute(db, cv, 2, "cvbncb", elements2);
		createRoute(db, cv, 3, "ertwertwr", elements3);
		createRoute(db, cv, 4, "errdade", elements4);
	}

	/**
	 * Creates a route, including its elements, in the database.
	 * 
	 * @param db
	 *            the database.
	 * @param cv
	 *            the content builder.
	 * @param id
	 *            the id of the route.
	 * @param keyword
	 *            the keyword of the route.
	 * @param elements
	 *            the elements of the route.
	 */
	public void createRoute(SQLiteDatabase db, ContentValues cv, int id,
			String keyword, int[] elements) {
		cv.clear();
		cv.put(ctx.getString(R.string.dbRouteId), id);
		cv.put(ctx.getString(R.string.dbRouteKeyword), keyword);
		db.insert(ctx.getString(R.string.dbRoute), null, cv);
		cv.clear();
		for (int element : elements) {
			cv.put(ctx.getString(R.string.dbRouteElementRouteId), id);
			cv.put(ctx.getString(R.string.dbRouteElementTaskId), element);
			db.insert(ctx.getString(R.string.dbRouteElement), null, cv);
		}
	}

	/**
	 * Upgrades the database by deleting all tables and creating it from
	 * scratch.
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		Log.e("DB", "onUpgrade");
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbTask));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbRoute));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbRouteElement));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbGameState));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbAlternative));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbQuestion));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbHint));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbGPS));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbShazam));
		onCreate(db);
	}

	public void resetDatabase(SQLiteDatabase db) {
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbTask));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbRoute));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbRouteElement));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbGameState));
		db.execSQL("DROP TABLE IF EXISTS "
				+ ctx.getString(R.string.dbAlternative));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbQuestion));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbHint));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbGPS));
		db.execSQL("DROP TABLE IF EXISTS " + ctx.getString(R.string.dbShazam));
		onCreate(db);
	}

	/**
	 * Creates a GPS task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - id of the image
	 * @param penalty
	 *            - penalty incurred when answering wrong
	 * @param latitude
	 *            - latitude component of the gps coordinate
	 * @param longitude
	 *            - longitude component of the gps coordinate
	 * @param radius
	 *            - the range the user needs to get within of the gps
	 *            coordinates
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task created. -1 if failed.
	 */
	public long createGPSTask(SQLiteDatabase db, String description,
			int imageId, int penalty, String latitude, String longitude,
			int radius, String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), "");
		cv.put(ctx.getString(R.string.dbTaskType), Task.GPS_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);
		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);

			cv.clear();
			cv.put(ctx.getString(R.string.dbGPSLatitude), latitude);
			cv.put(ctx.getString(R.string.dbGPSLongitude), longitude);
			cv.put(ctx.getString(R.string.dbGPSRadius), radius);
			cv.put(ctx.getString(R.string.dbGPSTaskId), id);
			db.insertOrThrow(ctx.getString(R.string.dbGPS), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert GPS task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Creates a Shazam task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - id of the image
	 * @param solution
	 *            - the solution to the task
	 * @param penalty
	 *            - penalty incurred when answering wrong
	 * @param songId
	 *            - id of the song
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task created. -1 if failed
	 */
	public long createShazamTask(SQLiteDatabase db, String description,
			int imageId, String solution, int penalty, int songId,
			String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), solution);
		cv.put(ctx.getString(R.string.dbTaskType), Task.SHAZAM_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);
		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);

			cv.clear();
			cv.put(ctx.getString(R.string.dbShazamSong), songId);
			cv.put(ctx.getString(R.string.dbGPSTaskId), id);
			db.insertOrThrow(ctx.getString(R.string.dbShazam), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert GPS task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Creates a MultipleChoice task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - image id of the task
	 * @param solution
	 *            - solution to the task
	 * @param penalty
	 *            - penalty if wrong answer given
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task created. -1 if failed
	 */
	public long createMultipleChoiceTask(SQLiteDatabase db, String description,
			int imageId, String solution, int penalty, String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), solution);
		cv.put(ctx.getString(R.string.dbTaskType), Task.MULTIPLE_CHOICE_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);
		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert MultipleChoice task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Adds a multiple choice alternative to a task
	 * 
	 * @param db
	 *            - the database
	 * @param taskId
	 *            - the id of the target task
	 * @param text
	 *            - the text the alternative will be displayed as
	 * @param isCorrect
	 *            - if the alternative is the solution to the task
	 */
	public void addAlternative(SQLiteDatabase db, long taskId, String text,
			boolean isCorrect) {
		if (taskId < 0) {
			Log.e("Database", "Could not add alternative. Invalid task ID: "
					+ taskId);
			return;
		}
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbAlternativeAlternative), text);
		cv.put(ctx.getString(R.string.dbAlternativeTaskId), (int) taskId);
		cv.put(ctx.getString(R.string.dbAlternativeIsCorrect), isCorrect ? 1
				: 0);
		try {
			db.insert(ctx.getString(R.string.dbAlternative), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert alternative (Task ID: "
					+ taskId);
			Log.e("Database", e.getMessage());
		}
	}

	/**
	 * Creates a Open task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - id of the image
	 * @param solution
	 *            - the solution of the task
	 * @param penalty
	 *            - penalty given if answered wrong
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task. -1 if failed
	 */
	public long createOpenTask(SQLiteDatabase db, String description,
			int imageId, String solution, int penalty, String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), solution);
		cv.put(ctx.getString(R.string.dbTaskType), Task.OPEN_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);
		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert Checkbox task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Creates a Barcode scanning task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - id of the image
	 * @param solution
	 *            - the solution of the task
	 * @param penalty
	 *            - penalty given if answered wrong
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task. -1 if failed
	 */
	public long createBarcodeTask(SQLiteDatabase db, String description,
			int imageId, String solution, int penalty, String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), solution);
		cv.put(ctx.getString(R.string.dbTaskType), Task.BARCODE_SCANNING_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);
		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert barcode task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Creates a checkbox task in the database.
	 * 
	 * @param db
	 *            - the database
	 * @param description
	 *            - description of the task
	 * @param imageId
	 *            - id of the image
	 * @param penalty
	 *            - penalty given if answered wrong
	 * @param bypassCode
	 *            - the bypass/failsafe code that is required to auto-complete
	 *            this task, reset it's hints used and wrong answers given
	 * @return id of the task. -1 if failed
	 */
	public long createCheckBoxTask(SQLiteDatabase db, String description,
			int imageId, int penalty, String bypassCode) {
		long id = -1;
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbTaskDescription), description);
		cv.put(ctx.getString(R.string.dbTaskCompleted), 0);
		cv.put(ctx.getString(R.string.dbTaskImage), imageId);
		cv.put(ctx.getString(R.string.dbTaskSolution), "");
		cv.put(ctx.getString(R.string.dbTaskType), Task.CHECKBOX_TASK);
		cv.put(ctx.getString(R.string.dbTaskPenalty), penalty);
		cv.put(ctx.getString(R.string.dbTaskWrongAnswers), 0);
		cv.put(ctx.getString(R.string.dbTaskBypassCode), bypassCode);

		try {
			id = db.insertOrThrow(ctx.getString(R.string.dbTask), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert Checkbox task");
			Log.e("Database", e.getMessage());
		}
		return id;
	}

	/**
	 * Adds a question to a task.
	 * 
	 * @param db
	 *            - the database
	 * @param taskId
	 *            - id of the target task
	 * @param text
	 *            - the question text
	 */
	public void addQuestion(SQLiteDatabase db, long taskId, String text) {
		if (taskId < 0) {
			Log.e("Database", "Could not add question. Invalid task ID: "
					+ taskId);
			return;
		}
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbQuestionQuestion), text);
		cv.put(ctx.getString(R.string.dbQuestionTaskId), (int) taskId);
		try {
			db.insert(ctx.getString(R.string.dbQuestion), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert question (Task ID: " + taskId);
			Log.e("Database", e.getMessage());
		}
	}

	/**
	 * Adds a hint to a task.
	 * 
	 * @param db
	 *            - the database
	 * @param taskId
	 *            - id of the target task
	 * @param text
	 *            - hint text
	 * @param imageId
	 *            - id of the image
	 * @param songId
	 *            - id of the song
	 * @param penalty
	 *            - penalty given if hint is used
	 */
	public void addHint(SQLiteDatabase db, long taskId, String text,
			int imageId, int songId, int penalty) {
		if (taskId < 0) {
			Log.e("Database", "Could not add hint. Invalid task ID: " + taskId);
			return;
		}
		ContentValues cv = new ContentValues();
		cv.put(ctx.getString(R.string.dbHintTaskId), (int) taskId);
		cv.put(ctx.getString(R.string.dbHintDescription), text);
		cv.put(ctx.getString(R.string.dbHintImage), imageId);
		cv.put(ctx.getString(R.string.dbHintSong), songId);
		cv.put(ctx.getString(R.string.dbHintPenalty), penalty);
		cv.put(ctx.getString(R.string.dbHintUsed), 0);
		try {
			db.insert(ctx.getString(R.string.dbHint), null, cv);
		} catch (SQLException e) {
			Log.e("Database", "Could not insert hint (Task ID: " + taskId);
			Log.e("Database", e.getMessage());
		}
	}
}