package acg.prototype.ui;

import acg.prototype.R;
import acg.prototype.content.Hint;
import acg.prototype.content.Task;
import acg.prototype.util.Helper;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.media.MediaPlayer;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * The abstract class defining the structure of all Task UIs. 
 * Also contains a number of methods to be used by those UIs 
 * to build common elements.
 */
public abstract class TaskUI {

	/**
	 * The task to be solved.
	 */
	protected Task task;
	
	/**
	 * Array of task IDs representing the route.
	 */
	protected int[] taskIds;

	/**
	 * The activity requesting this UI.
	 */
	protected Activity activity;
	
	/**
	 * The top-level layout container that contains all other layouts and is scrollable.
	 */
	protected ScrollView scroll;
	
	/**
	 * The layout container contained in the scrollview.
	 */
	protected LinearLayout layout;
	
	/**
	 * An inner layout contained in <b>layout</b>. Contains hint button and ok button.
	 */
	protected LinearLayout innerLowerLayout;
	
	/**
	 * An inner layout contained in <b>layout</b>. Contains text views representing hints.
	 */
	protected LinearLayout hintLayout;

	/**
	 * The hint button
	 */
	protected Button hintButton;

	/**
	 * Creates a new instance of the UI handler.
	 * 
	 * @param task
	 *            the task to be solved.
	 */
	public TaskUI(Task task, int[] taskIds, Activity activity) {
		this.task = task;
		this.activity = activity;
		this.taskIds = taskIds;
	}

	/**
	 * Called when a task is solved and the Android activity shall finish
	 * successfully.
	 */
	public void solveTask() {
		task.setCompleted(true);
		Intent i = new Intent();
		i.putExtra("back", 0);
		activity.setResult(Activity.RESULT_OK, i);
		activity.finish();
	}

	/**
	 * Creates and returns the UI of the task.
	 * 
	 * @return the UI of the task.
	 */
	public abstract View createView();
	
	/**
	 * Sets up and adds viewable hints to the interface.
	 */
	public void addHintView() {
		if (!task.hasHints()) return;
		
		hintLayout = new LinearLayout(activity);
		hintLayout.setOrientation(LinearLayout.VERTICAL);
		hintLayout.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
		hintLayout.setGravity(Gravity.CENTER_HORIZONTAL);
		
		// check for used hints and build them if any found
		Hint[] hints = task.getUsedHints();
		if (hints.length!=0) {
			for (int i=0; i<hints.length; i++) buildHint(hints[i]);
		}
		layout.addView(hintLayout);
	}
	
	/**
	 * Adds the hint button to the interace.
	 */
	public void addHintButton() {
		if (!task.hasHints()) return;
		if (task.getNextUnusedHint() != null) {
			hintButton = new Button(activity);
			hintButton.setText(R.string.hint);
			hintButton.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1.0f));
			hintButton.setOnClickListener(new OnClickListener() {
				public void onClick(View v) {
					//Ask the user to confirm answer
			        new AlertDialog.Builder(activity)
			        .setIcon(android.R.drawable.ic_dialog_alert)
			        .setTitle(R.string.confirmHint)
			        .setMessage(activity.getString(R.string.confirmHintText)+" "+task.getNextUnusedHint().getPenalty()+ " minutes")
			        .setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
	
			            public void onClick(DialogInterface dialog, int which) {
			            	Hint hint = task.getNextUnusedHint();
			            	if (hint!=null) {
			            		buildHint(hint);
			            		task.setHintUsed(hint);
				            	if (task.getRemainingHints()==0) innerLowerLayout.removeView(hintButton);
			            	}
			            }
			        })
			        .setNegativeButton(R.string.no, null)
			        .show();
				}
			});
			innerLowerLayout.addView(hintButton);
		}
	}
	
	/**
	 * Adds an ok button to the task layout, as well as a confirmation popup of type DialogInterface
	 */
	public void addOkButton() {
		Button okButton = new Button(activity);
		okButton.setText(R.string.ok);
		okButton.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT, 1.0f));
		okButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				//Ask the user to confirm answer
		        new AlertDialog.Builder(activity)
		        .setIcon(android.R.drawable.ic_dialog_alert)
		        .setTitle(R.string.confirmAnswer)
		        .setMessage(activity.getString(R.string.confirmAnswerText)+" "+ task.getPenalty()+" minutes.")
		        .setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {

		            public void onClick(DialogInterface dialog, int which) {
		        			if (task.validateTask(getGivenAnswer()))
		        				solveTask();
		        			else {
		        				task.incWrongAnswers();
			            		Toast.makeText(activity, R.string.wrongAnswer, Toast.LENGTH_LONG).show();
			            	}
		            }
		        })
		        .setNegativeButton(R.string.no, null)
		        .show();
			}
		});
		innerLowerLayout.addView(okButton);
		
	}
	
	/**
	 * Adds an image to <b>layout</b> if the task has an image.
	 */
	protected void addImage(){
		Log.e("imageId", String.valueOf(task.getImageId()));
		if(task.getImageId() != 0){
			ImageView image = new ImageView(activity);
			image.setImageResource(Helper.getImage(task.getImageId()));
			image.setAdjustViewBounds(true);
			layout.addView(image);
		}
	}
	
	/**
	 * Adds a description TextView to <b>layout</b>.
	 */
	protected void addDescription(){
		TextView description = new TextView(activity);
		description.setText(task.getDescription());
		description.setTextColor(Color.YELLOW);
		layout.addView(description);
	}
	
	/**
	 * Initializes the layout containers for the task, which is a ScrollView containing a LinearLayout.
	 */
	protected void createLayout(){
		scroll = new ScrollView(activity);
		layout = new LinearLayout(activity);
		layout.setOrientation(LinearLayout.VERTICAL);
		scroll.addView(layout);
		TextView progress = new TextView(activity);
		int current = 0;
		for (int i=0; i<taskIds.length; i++) {
			if (taskIds[i] == task.getId()) current = i+1;
		}
		progress.setText("Task "+current+"/"+taskIds.length);
		layout.addView(progress);
	}
	
	/**
	 * Returns the answer given by the user.
	 * @return Object containing the answer given by the user.
	 */
	public abstract Object getGivenAnswer();
	
	/**
	 * Sets up the lower interface used for hint button, ok button and the gps button.
	 */
	protected void createLowerInterface() {
		innerLowerLayout = new LinearLayout(activity);
		innerLowerLayout.setOrientation(LinearLayout.HORIZONTAL);
		innerLowerLayout.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
		innerLowerLayout.setPadding(1, 10, 1, 1);
	}
	
	/**
	 * Builds the view for representing the hint
	 * @param hint - hint to be shown
	 */
	private void buildHint(Hint hint) {
		if(!hint.getText().equals("")){
    		TextView t = new TextView(activity);
    		t.setText("- "+hint.getText());
    		t.setTextColor(Color.GREEN);
        	t.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
        	hintLayout.addView(t);
		}
		if(hint.getImageId() != 0){
			ImageView image = new ImageView(activity);
			image.setImageResource(Helper.getImage(hint.getImageId()));
			image.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.WRAP_CONTENT));
			image.setAdjustViewBounds(true);
			hintLayout.addView(image);
		}
		if(hint.getSongId() != 0){
			Button playButton = new Button(activity);
			playButton.setBackgroundDrawable(activity.getResources().getDrawable(R.drawable.playbutton));
			LayoutParams playButtonParams = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT, 1.0f);
			playButtonParams.setMargins(1, 10, 1, 10);
			playButton.setLayoutParams(playButtonParams);
			final int songId = hint.getSongId();
			final MediaPlayer mp = MediaPlayer.create(activity, Helper.getSong(songId));
			playButton.setOnClickListener(new OnClickListener() {

				public void onClick(View v) {
				    if(mp.isPlaying()){
				    	mp.pause();
				    }else{
				    	mp.start();
				    }
				}
			});
			hintLayout.addView(playButton);
		}
	}
	
	/**
	 * Builds the question(s) view and adds them to layout
	 */
	public void buildQuestions() {
		String[] questions = task.getQuestions();
		for(int i = 0; i < questions.length; i++){
			TextView question = new TextView(activity);
			question.setText(questions[i]);
			layout.addView(question);
		};
	}
}