package acg.prototype.util;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;

/**
 * A helper class for retrieving GPS coordinates.
 * 
 * LocationListener - listens for GPS updates.
 */
public class GPSHandler implements LocationListener{

	/**
	 * LocationManager for retrieving GPS coordinates.
	 */
	private LocationManager locationManager;
	
	/**
	 * The location the user should find in the given task.
	 */
	private Location pointLocation; 
	
	/**
	 * The radius around the fixed location. If the user is inside the radius, the task is complete.
	 */
	private float radius;
	
	/**
	 * Tells if the user is inside the range of the location.
	 */
	private boolean insideTaskRange;
	
	/**
	 * True if onLocationChanged method has been called, false otherwise.
	 */
	private boolean onLocationChanged;
	 
	
	
	/**
	 * Constructs a new GPSHandler.
	 * 
	 * @param context - the current activity.
	 */
	public GPSHandler(Context context){
		locationManager = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
		pointLocation = new Location("");
		radius = 0;
		insideTaskRange = false;
		onLocationChanged = false;
	}
	
	/**
	 * Is fired when a new GPS location has been retrieved. If the user is in range of the location, insideTaskRange is set to true.
	 */
	public void onLocationChanged(Location location) {
		float distance = location.distanceTo(pointLocation);
		radius = radius + location.getAccuracy();
		if(distance <= radius){
			insideTaskRange = true;
		}else{
			insideTaskRange = false;
		}
		onLocationChanged = true;
		stopUpdates();
	}
	
	
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub
		
	}
	
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub
		
	}

	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub
		
	}
	
	
	/**
	 * Returns true if the user is in range of the location, false otherwise.
	 * 
	 * @return true if the user is in range of the location, false otherwise.
	 */
	public boolean getInsideTaskRange(){
		return insideTaskRange;
	}
	
	/**
	 * Starts GPS updates.
	 */
	public void startUpdates(){
		locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);
	}
	
	/**
	 * Stops GPS updates.
	 */
	public void stopUpdates(){
		locationManager.removeUpdates(this);
	}

	/**
	 * Sets the location that the user should find.
	 * 
	 * @param latitude - the latitude of the location.
	 * @param longitude - the longitude of the location.
	 * @param radius - the radius around the location.
	 */
	public void setCoordinates(double latitude, double longitude, float radius) {
		pointLocation.setLatitude(latitude);
		pointLocation.setLongitude(longitude);
		this.radius = radius;
		
	}

	/**
	 * Sets onLocationChanged.
	 * 
	 * @param onLocationChanged
	 */
	public void setOnLocationChanged(boolean onLocationChanged) {
		this.onLocationChanged = onLocationChanged;
	}

	/**
	 * Returns true if onLocationChanged method has been called, false otherwise.
	 * 
	 * @return true if onLocationChanged method has been called, false otherwise.
	 */
	public boolean getOnLocationChanged() {
		return onLocationChanged;
	}
}